/* ******************************************************************

   class    : AbstractTool
   date     : Okt 05 1998
   compiler : javac (1.1.6)

******************************************************************* */

/* ******************************************************************
   import statements
******************************************************************* */

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.*;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import com.sun.java.swing.Action;
import com.sun.java.swing.AbstractAction;

import Utils.ResourceUtil;
import Messages.Message;

/**
 * A class that should be overwritten to implement tool plugins.
 *
 * @author puehringer edgar
 */
public abstract class AbstractTool extends Observable
  implements ActionListener {

  // class variable declarations
  /** Key to the project extensions for the getExtensions method. */
  protected static final String PROJECT = Const.PROJECT_EXT;
  /** Key to the source extensions for the getExtensions method. */
  protected static final String SOURCE = Const.SOURCE_EXT;
  /** Key to the object extensions for the getExtensions method. */
  protected static final String OBJECT = Const.OBJECT_EXT;
  /** Key to the runnable extensions for the getExtensions method. */
  protected static final String EXECUTABLE = Const.EXEC_EXT;
  /** Key to the source code template extensions for the getExtensions method. */
  protected static final String TEMPLATE = Const.TPL_EXT;

  // instance variable declarations
  /** The _Main resource bundle. */
  protected final ResourceBundle resources;
  /** Prefix for resource keys to get an unique name space. */
  protected final String keyPrefix;
  /** The parent frame for dialogs, etc. */
  protected Frame parent;
  private final Properties project;
  private Hashtable commands;

  // constructors
  /**
   * Create a new AbstractTool object.  Don't forget the
   * super call!
   */
  public AbstractTool(ResourceBundle resources, Properties project,
                      String keyPrefix) {
    this.resources = resources;
    this.project   = project;
    this.keyPrefix = keyPrefix;
    parent         = null;
  } // end constructor AbstractTool

  // class methods
  /**
   * Returns an instance of a derived class of AbstractTool.
   *
   * @param clazz The class name.
   * @param resources Resource bundle containing GUI labels, etc.
   * @param project Project related properties.
   * @param keyPrefix Prefix for resource keys to get an unique name space.
   */
  public static AbstractTool getInstance(String clazz,
                               ResourceBundle resources, Properties project,
                               String keyPrefix) {
    Object[] objects;
    Class[] classes;
    Class c;
    Constructor constr;
    AbstractTool tool = null;

    if (clazz == null) return null;
    objects    = new Object[3];
    objects[0] = resources;
    objects[1] = project;
    objects[2] = keyPrefix;
    classes    = new Class[3];
    classes[0] = java.util.ResourceBundle.class;
    classes[1] = java.util.Properties.class;
    classes[2] = java.lang.String.class;
    try {
      c      = Class.forName(clazz);
      constr = c.getConstructor(classes);
      tool   = (AbstractTool)constr.newInstance(objects);
    } catch (ClassNotFoundException excep) {
      excep.printStackTrace();
    } catch (IllegalAccessException excep) {
       excep.printStackTrace();
    } catch (InstantiationException excep) {
       excep.printStackTrace();
    } catch (NoSuchMethodException excep) {
       excep.printStackTrace();
    } catch (InvocationTargetException excep) {
       excep.printStackTrace();
    }
    return tool;
  } // end getInstance

  // methods
  /** Returns a project property. */
  protected String getProjectProperty(String key) {
    return project.getProperty(key);
  } // end getProjectProperty

  /** Returns a property from this tool. */
  protected String getToolProperty(String key) {
    return project.getProperty(keyPrefix + key);
  } // end getToolProperty

  /** Sets a property for this tool. */
  protected void putToolProperty(String key, String value) {
    key = keyPrefix + key;
    if (!value.equals(project.getProperty(key))) {
      project.put(key,value);
      project.put(Const.PROJECT_CHANGED,"" + true);
    } // end if
  } // end putToolProperty

  /** Returns the project directory. */
  protected String getProjectDirectory() {
    String str = project.getProperty(Const.PROJECT_DIR);

    Utils.Assert.assert(Utils.Assert.ENABLED &&
      (str != null),"Project directory not spezified");
    return ResourceUtil.urlToFile(str);
  } // end getProjectDirectory

  /** Returns an array containing all project files. */
  protected File[] getProjectFiles() {
    int numFiles  = Integer.parseInt(project.getProperty(Const.NUM_FILES));
    Vector vector = new Vector(numFiles > 0 ? numFiles : 1);
    File[] files;

    for (int i=0; i<numFiles; i++) {
      String name = project.getProperty(Const.FILE_PREFIX + i);

      if (name == null) {
        System.err.println("AbstractTool: Project file number wrong.");
        break;
      } // end if
      vector.addElement(new File(ResourceUtil.urlToFile(name)));
    } // end for
    numFiles = vector.size();
    files = new File[numFiles];
    for (int i=0; i<numFiles; i++) files[i] = (File)vector.elementAt(i);
    return files;
  } // end getProjectFiles

  /**
   * Returns an array containing all extensions of the
   * specified file type.
   *
   * @param key The file type.
   */
  protected String[] getExtensions(String key) {
    String str = project.getProperty(key);

    Utils.Assert.assert(Utils.Assert.ENABLED &&
      (str != null),"Resource for file extensions not found");
    return ResourceUtil.splitToWords(str);
  } // end getExtensions

  /** Returns an array containing all action commands of this tool. */
  protected abstract String[] getActionCommands();

  /**
   * Returns all actions of this tool.
   *
   * @see getActionCommands
   */
  public Action[] getActions() {
    Action[] actions;
    Enumeration enum;
    int i;

    if (commands == null) {
      String[] cmds = getActionCommands();

      commands = cmds.length > 0 ? new Hashtable(cmds.length) : new Hashtable();
      for (i=0; i<cmds.length; i++) {
        String key = keyPrefix + cmds[i];

        commands.put(key,new AbstractAction(key) {

          public void actionPerformed(ActionEvent e) {
            AbstractTool.this.actionPerformed(e);
          } // end actionPerformed

        }
        );
      } // end for
    } // end if
    actions = new Action[commands.size()];
    enum    = commands.keys();
    i       = 0;
    while (enum.hasMoreElements())
      actions[i++] = (Action)commands.get(enum.nextElement());
    // end while
    return actions;
  } // end getActions

  /** Sets an action of this tool enabled or disabled. */
  protected void setEnabled(String cmd, boolean enbl) {
    Action action = (Action)commands.get(keyPrefix + cmd);

    if (action != null) action.setEnabled(enbl);
  } // end setEnabled

  /** Sets an action of this tool checked or unchecked. */
  protected void setState(String cmd, boolean enbl) {
    Action action = (Action)commands.get(keyPrefix + cmd);

    if (action != null)
      action.putValue("checked",enbl ? Boolean.TRUE : Boolean.FALSE);
    // end if
  } // end setState

  /** Returns true if an action of this tool is checked. */
  protected boolean getState(String cmd) {
    Action action = (Action)commands.get(keyPrefix + cmd);

    return ((action != null) &&
            ((Boolean)action.getValue("checked")).booleanValue());
  } // end getState

  /** Sets the parent frame for this tool. */
  public void setParentFrame(Frame frame) {
    parent = frame;
  } // end setParentFrame

  /** Posts a message to all observers of this observable. */
  public void postMessage(Message msg) {
    setChanged();
    notifyObservers(msg);
  } // end postMessage

} // end AbstractTool

/* ******************************************************************
   end of file
******************************************************************* */
