/* ******************************************************************

   compiler   : javac (jdk 1.1.6)
   class      : FileManagerPanel
   date       : 15.07.98

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.URL;

import com.sun.java.swing.*;
import com.sun.java.swing.text.TextAction;
import com.sun.java.swing.event.ListSelectionListener;

import Utils.ResourceUtil;

/* ******************************************************************
   top level class declaration
*********************************************************************/

/**
 * A panel that displays files and tracks directory changes.
 *
 * @author Puehringer Edgar
 */
class FileManagerPanel extends JPanel implements FilenameFilter {

  // class variable declarations
  /** Key to the file manager label text */
  public static final String FILEMAN_LABEL = "fmLabelText";
  /** Identifier of the up dir action */
  public static final String UPDIR_ACTION = "up-dir";
  /** Identifier of the refresh list action */
  public static final String REFRESH_ACTION = "refresh";

  // instance variable declarations
  private ResourceBundle resources;
  private Properties project;
  private Action dblClkAction;
  private String labelText;
  private JLabel label;
  private boolean enabled = false;
  private JScrollPane scroller;
  private DefaultListModel model    = new DefaultListModel();
  private DefaultListModel tmpModel = new DefaultListModel();
  private JList list                = new JList(model);
  private File path;
  private String[] sourceExt;
  /** Actions defined by the FileManagerPanel class. */
  private UpdirAction updirAction     = new UpdirAction();
  private RefreshAction refreshAction = new RefreshAction();
  private Action[] defaultActions = {
    updirAction,
    refreshAction
  };

  // constructors
  FileManagerPanel(ResourceBundle resources, Properties project) {
    this(resources,project,null);
  } // end constructor FileManagerPanel

  FileManagerPanel(ResourceBundle resources, Properties project,
                   Action dblClkAction) {
    this.resources    = resources;
    this.project      = project;
    this.dblClkAction = dblClkAction;

    String str = this.project.getProperty(Const.PROJECT_DIR);

    if (str.startsWith("file:/")) str = ResourceUtil.urlToFile(str);
    path      = new File(str);
    str       = this.project.getProperty(Const.SOURCE_EXT);
    sourceExt = ResourceUtil.splitToWords(str);
    setLayout(new BorderLayout());
    labelText = ResourceUtil.getResourceString(resources,FILEMAN_LABEL);
    labelText = labelText == null ? "" : labelText + " ";
    label = new JLabel(labelText + path);
    label.setToolTipText(path.toString());
    add("North",label);
    scroller = new JScrollPane(list);

    URL url = ResourceUtil.getResource(this,this.resources,Const.DIR_IMAGE);
    ImageIcon dirImage = url == null ? null : new ImageIcon(url);
    url = ResourceUtil.getResource(this,this.resources,Const.DOC_IMAGE);
    ImageIcon docImage = url == null ? null : new ImageIcon(url);
    list.setCellRenderer(new FileCellRenderer(dirImage,docImage,this));
    list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    MouseListener mouseListener = new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        if ((e.getClickCount() == 2) && !list.isSelectionEmpty()) {
          File file = (File)list.getSelectedValue();

          if (file.isDirectory()) {
            path = file;
            updateList();
            label.setText(labelText + path);
            label.setToolTipText(path.toString());
            if (!updirAction.isEnabled()) updirAction.setEnabled(true);
          } else if (FileManagerPanel.this.dblClkAction != null) {
            ActionEvent ae = new ActionEvent(list,ActionEvent.ACTION_PERFORMED,"");
            FileManagerPanel.this.dblClkAction.actionPerformed(ae);
          } else {
            Toolkit.getDefaultToolkit().beep();
          } // end if
        } // end if
      } // end mouseClicked
    }; // end anonymous class
    list.addMouseListener(mouseListener);
    updateList();
    add("Center",scroller);
    enabled = true;
  } // end constructor FileManagerPanel

  // class methods
  /** Forces update of the enclosing scrollers on list model change. */
  private static void setModel(JList l,DefaultListModel m) {
    int size = m.getSize();

    if (size > 0) {
      Object obj = m.lastElement();

      m.removeElementAt(size - 1);
      l.setModel(m);
      m.add(size - 1,obj);
    }
    else {
      l.setModel(m);
    } // end if
  } // end setModel

  // methods
  /**
   * Enables (or disables) the component.
   */
  public void setEnabled(boolean b) {
    if (b == enabled) return;
    if (b) {
      String str = project.getProperty(Const.PROJECT_DIR);
      File p;

      if (str.startsWith("file:/")) str = ResourceUtil.urlToFile(str);
      p = new File(str);
      if (!p.equals(path)) {
        path = p;
        updateList();
      } // end if
    } else if (!list.isSelectionEmpty()) {
      list.clearSelection();
    } // end if
    updirAction.setEnabled((path.getParent() != null) && b);
    refreshAction.setEnabled(b);
    enabled = b;
  } // end setEnabled

  /**
   * Returns true if the component is enabled, false otherwise.
   */
  public boolean isEnabled() {
    return enabled;
  } // end isEnabled

  /**
   * Add a listener to the list that's notified each time a
   * change to the selection occurs.
   */
  public void addListSelectionListener(ListSelectionListener listener) {
    list.addListSelectionListener(listener);
  } // end addListSelectionListener

  /**
   * Remove a listener from the list that's notified each time a change
   * to the selection occurs.
   */
  public void removeListSelectionListener(ListSelectionListener listener) {
    list.removeListSelectionListener(listener);
  } // end removeListSelectionListener

  /**
   * Returns an array containing the selected list items.
   */
  public Object[] getSelectedValues() {
    return list.getSelectedValues();
  } // end getSelectedValues

  /**
   * Returns the current selected path.
   */
  public File getPath() {
    return path;
  } // end getPath

  void updateList() {
    String str;
    String[] files = path.list();
    int numDirs    = 0;
    int numDocs    = 0;

    // reload the source extensions
    str       = project.getProperty(Const.SOURCE_EXT);
    sourceExt = ResourceUtil.splitToWords(str);
    // creating a new model is fast than changing the current one !
    setModel(list,tmpModel);

    model.removeAllElements();
    for (int i=0; i<files.length; i++) {
      File f = new File(path,files[i]);

      if (f.isDirectory()) {
        model.insertElementAt(f,numDirs);
        numDirs++;
      } else if (accept(path,files[i])) {
        model.insertElementAt(f,numDirs + numDocs);
        numDocs++;
      } else {
        model.addElement(f);
      } // end if
    } // end for
    setModel(list,model);
    if (enabled) {
      list.repaint();
      scroller.validate();
    } // end if
  } // end updateList;

  /**
   * Fetch the list of actions supported by this panel.
   */
  public Action[] getActions() {
    return defaultActions;
  } // end getActions

  //--------- FilenameFilter ----------------------------------------
  public boolean accept(File dir, String name) {
    for (int i=0; i<sourceExt.length; i++)
      if (name.endsWith(sourceExt[i])) return true;
    // end for
    return false;
  } // end accept

  // inner classes
  /** Sets the current displayed directory to the parent directory. */
  class UpdirAction extends AbstractAction {

    // constructors
    UpdirAction() {
      super(UPDIR_ACTION);

    } // end constructor UpdirAction

    // instance methods
    public void actionPerformed(ActionEvent e) {
      String parent = path.getParent();

      if (parent == null) {
        Toolkit.getDefaultToolkit().beep();
        UpdirAction.this.setEnabled(false);
        return;
      } // end if
      path = new File(parent);
      if (path.getParent() == null) UpdirAction.this.setEnabled(false);
      updateList();
      label.setText(labelText + path);
      label.setToolTipText(path.toString());
    } // end actionPerformed

  } // end UpdirAction

  /** Refreshes the current displayed directory list. */
  class RefreshAction extends AbstractAction {

    // constructors
    RefreshAction() {
      super(REFRESH_ACTION);
   } // end constructor RefreshAction

    // instance methods
    public void actionPerformed(ActionEvent e) {
      updateList();
    } // end actionPerformed

  } // end RefreshAction

  class FileCellRenderer extends JLabel implements ListCellRenderer {

    // instance variable declarations
    private boolean focused = false;
    private Icon dirIcon;
    private Icon docIcon;
    private FilenameFilter filter;

    // constructors
    public FileCellRenderer (Icon dirIcon, Icon docIcon,
                             FilenameFilter filter) {
      this.dirIcon = dirIcon;
      this.docIcon = docIcon;
      this.filter  = filter;
      setOpaque(true);
    } // end constructor FileCellRenderer

    public Component getListCellRendererComponent(
        JList list, Object value, int index,
        boolean isSelected, boolean cellHasFocus) {
      File f = (File)value;
      if (value == null) {
        setText("");
        setIcon(null);
      }
      else {
        setText(f.getName());
        if (f.isDirectory()) {
          setIcon(dirIcon);
        } else if (filter.accept(new File(f.getParent()),f.getName())) {
          setIcon(docIcon);
        } else {
          setIcon(null);
        } // end if
      } // end if
      setBackground (isSelected ?
        SystemColor.textHighlight :
        SystemColor.window);
      setForeground (isSelected ?
        SystemColor.textHighlightText :
        SystemColor.windowText);
      return this;
    } // end getListCellRendererComponent

    public Dimension getPreferredSize() {
      Dimension dim = super.getPreferredSize();
      dim.width += 15;  // widen
      return dim;
    } // end getPreferredSize

  } // end FileCellRenderer

} // end FileManagerPanel

/* ******************************************************************
   end of file
*********************************************************************/
