/* ******************************************************************

   class    : MenuEditorPanel
   date     : Sep 21 1998
   compiler : javac (1.1.6)

******************************************************************* */

/* ******************************************************************
   import statements
******************************************************************* */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import com.sun.java.swing.*;
import com.sun.java.swing.text.TextAction;
import com.sun.java.swing.text.Document;
import com.sun.java.swing.text.PlainDocument;

import Utils.ResourceUtil;
import Utils.FrameUtil;
import Utils.StringUtil;
import CustCtrl.BaseDialog;
import CustCtrl.SmartTextField;
import MenuPreview.JMenuTestFrame;
import MenuPreview.MenuTestFrame;

/**
 * An editor panel to build up menu editors.
 *
 * @author puehringer edgar
 */
public class MenuEditorPanel extends CodepadPanel {

  // class variable declarations
  /** Key to the menubar resource. */
  private static final String MENUBAR = "meMenubar";

  /** Key to the toolbar resource. */
  private static final String TOOLBAR = "meToolbar";

  /** Key to the status line resource */
  private static final String STATUS_LINE = "meStatusLine";

  /** The help topic to get help for this panel. */
  private static final String HELP_TOPIC = "menu-editor";

  /** Identifier of the insert menu item action. */
  public static final String INSITEM_ACTION = "insert-item";

  /** Identifier of the insert menu action. */
  public static final String INSMENU_ACTION = "insert-menu";

  /** Identifier of the insert menubar action. */
  public static final String INSMENUBAR_ACTION = "insert-menubar";

  /** Identifier of the insert toolbar action. */
  public static final String INSTOOLBAR_ACTION = "insert-toolbar";

  /** Identifier of the preview menu action. */
  public static final String PREVIEW_ACTION = "preview";

  // Keys to the insert menu item dialog
  public final static String ITEMDLG_TITLE        = "meItemDlg.title";
  public final static String ITEMDLG_BUTTONS      = "meItemDlg.buttons";
  public final static String ITEMDLG_KEYLABEL     = "meItemDlg.keyLabel";
  public final static String ITEMDLG_TYPELABEL    = "meItemDlg.typeLabel";
  public final static String ITEMDLG_CHOICE       = "meItemDlg.choice";
  public final static String ITEMDLG_LABELLABEL   = "meItemDlg.labelLabel";
  public final static String ITEMDLG_ACTIONLABEL  = "meItemDlg.actionLabel";
  public final static String ITEMDLG_ACCELLABEL   = "meItemDlg.accelLabel";
  public final static String ITEMDLG_CTRLLABEL    = "meItemDlg.ctrlLabel";
  public final static String ITEMDLG_ALTLABEL     = "meItemDlg.altLabel";
  public final static String ITEMDLG_SHIFTLABEL   = "meItemDlg.shiftLabel";
  public final static String ITEMDLG_IMAGELABEL   = "meItemDlg.imageLabel";
  public final static String ITEMDLG_TOOLTIPLABEL = "meItemDlg.tooltipLabel";
  public final static String ITEMDLG_WRONGPATHMSG = "meItemDlg.wrongPathMsg";

  // Keys to the insert menu dialog
  public final static String MENUDLG_TITLE        = "meMenuDlg.title";
  public final static String MENUDLG_BUTTONS      = "meMenuDlg.buttons";
  public final static String MENUDLG_KEYLABEL     = "meMenuDlg.keyLabel";
  public final static String MENUDLG_LABELLABEL   = "meMenuDlg.labelLabel";
  public final static String MENUDLG_SEPARBTN     = "meMenuDlg.separBtn";

  // Keys to the insert menubar dialog
  public final static String MENUBARDLG_TITLE = "meMenubarDlg.title";

  // Keys to the insert toolbar dialog
  public final static String TOOLBARDLG_TITLE = "meToolbarDlg.title";

  // Keys to the preview menu dialog
  public final static String PREVIEWDLG_TITLE      = "mePreviewDlg.title";
  public final static String PREVIEWDLG_BUTTONS    = "mePreviewDlg.buttons";
  public final static String PREVIEWDLG_MENULABEL  = "mePreviewDlg.menuLabel";
  public final static String PREVIEWDLG_TOOLLABEL  = "mePreviewDlg.toolLabel";
  public final static String PREVIEWDLG_SWINGLABEL = "mePreviewDlg.swingLabel";
  public final static String PREVIEWDLG_IMAGELABEL = "mePreviewDlg.imageLabel";

  // Keys to the swing menu test frame
  public final static String SWINGFRAME_TITLE = "meSwingFrame.title";

  // Keys to the AWT menu test frame
  public final static String AWTFRAME_TITLE = "meAWTFrame.title";

  /** Key to the syntax coloring class. */
  public final static String COLORING_PROPERTY = "propertiesColoringClass";
  /** Key to the menu editor default file extension. */
  public final static String MENU_EXT = "menuExtension";

  private static final String ABOUTDLG_PREFIX   = "meAboutDlg.";

  // instance variable declarations
  /** The menu key to display in the preview dialog. */
  private String menuKey = "";
  /** True if swing controls are enabled in the menu preview. */
  private boolean useSwing = true;
  /** True if images should be displayed in the swing menu items. */
  private boolean useImage = false;
  /** The toolbar key to display in the preview dialog. */
  private String toolbarKey = "";
  /** The additional supported actions of this editor. */
  private Action[] additionalActions;
  /** Contains the file extensions for files containing menu definitions. */
  private String[] menuExt;

  // constructors
  /** Create a new menu editor panel. */
  public MenuEditorPanel(Frame frame, ResourceBundle res, Properties desk,
                      Properties project) {
    super(frame,res,desk,project);
    init();
  } // end constructor MenuEditorPanel

  /**
   * Create a new menu editor panel.
   *
   * @param externalActions A list of additionally supported actions.
   */
  public MenuEditorPanel(Frame frame, ResourceBundle res, Properties desk,
                      Properties project, Action[] externalActions) {
    super(frame,res,desk,project,externalActions);
    init();
  } // end constructor MenuEditorPanel

  /**
   * Create a new menu editor panel.
   *
   * @param externalActions A list of additionally supported actions.
   * @param file A file that will be loaded at creation of this panel.
   */
  public MenuEditorPanel(Frame frame, ResourceBundle res, Properties desk,
                      Properties project, Action[] externalActions,
                      File file) {
    super(frame,res,desk,project,externalActions,file);
    init();
  } // end constructor MenuEditorPanel

  // class methods
  /* DEBUG ON */
  public static void main(String[] args) {
    ResourceBundle res = null;
    Properties desktop = new Properties();
    Properties project = new Properties();
    File file          = null;

    project.put(Const.PROJECT_DIR,
                ResourceUtil.fileToURL(System.getProperty("user.dir")));
    try {
      res = ResourceBundle.getBundle("_Main",Locale.getDefault());
    } catch (MissingResourceException mre) {
      System.err.println("MenuEditorPanel: _Main.properties not found");
      System.exit(1);
    }
    // Load the desktop settings
    try {
      file = new File("Desktop.properties");
    } catch (NullPointerException npe) {
      System.err.println("MenuEditorPanel: Desktop.properties not found");
      System.exit(2);
    }
    if (file.exists()) try {
      FileInputStream fis = new FileInputStream(file);

      desktop.load(fis);
      fis.close();
    } catch (IOException excep) {
      excep.printStackTrace();
    }

    JFrame frame               = new JFrame();
    Container cp               = frame.getContentPane();
    MenuEditorPanel menuEditor = new MenuEditorPanel(frame,res,desktop,project);

    frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    cp.setLayout(new BorderLayout());
    cp.add("Center",menuEditor);
    frame.pack();
    frame.setSize(500,600);
    frame.show();
  } // end main
  /* DEBUG OFF */

  // methods
  /** Fetch the list of actions supported by this editor. */
  public Action[] getActions() {
    if (additionalActions == null) {
      additionalActions    = new Action[5];
      additionalActions[0] = new InsItemAction();
      additionalActions[1] = new InsMenuAction();
      additionalActions[2] = new InsMenubarAction();
      additionalActions[3] = new InsToolbarAction();
      additionalActions[4] = new PreviewAction();
    } // end if
    return TextAction.augmentList(additionalActions,super.getActions());
  } // end getActions

  /** Returns the key to the menu bar. */
  protected String getMenuKey() {
    return MENUBAR;
  } // end getMenuKey

  /** Returns the key to the tool bar. */
  protected String getToolbarKey() {
    return TOOLBAR;
  } // end getToolbarKey

  /** Returns the key to the status line text. */
  protected String getStatuslineKey() {
    return STATUS_LINE;
  } // end getStatuslineKey

  /** Returns the help topic for this panel. */
  protected String getHelpTopic() {
    return HELP_TOPIC;
  } // end getHelpTopic

  /** Returns the key to the about dialog resource prefix. */
  protected String getAboutDlgKey() {
    return ABOUTDLG_PREFIX;
  } // end getAboutDlgKey

  /** Create an empty document. */
  protected Document createDocument() {
    String str;

    str = ResourceUtil.getResourceString(resources,COLORING_PROPERTY);
    return createDocument(str);
  } // end createDocument

  /**
   * Returns a sorted array of strings containing all keys
   * of this editors workload.
   */
  protected String[] getWorkloadKeys() {
    ByteArrayOutputStream baos = new ByteArrayOutputStream(4096);
    OutputStreamWriter osw     = new OutputStreamWriter(baos);
    ByteArrayInputStream bais;
    Properties prop;
    Vector v;
    Enumeration enum;
    String[] result = null;
    Document doc;
    Object obj;

    try {
      doc = getEditor().getDocument();
      obj = doc.getProperty(CodeEditorKit.CHANGED_PROPERTY);
      getEditor().write(osw);
      // avoid, that the document is marked as not changed on reading it
      doc.putProperty(CodeEditorKit.CHANGED_PROPERTY,obj);
      osw.close();
      baos.close();
      bais = new ByteArrayInputStream(baos.toByteArray());
      osw  = null;
      baos = null;
      prop = new Properties();
      prop.load(bais);
      bais.close();
      bais = null;
      enum = prop.keys();
      v    = prop.size() == 0 ? new Vector() : new Vector(prop.size());
      while (enum.hasMoreElements()) {
        String str = enum.nextElement().toString();
        int pos    = 0;

        if (str.endsWith(MenuPreview.Const.LABEL_SUFFIX)) {
          str = str.substring(0,str.length() -
                              MenuPreview.Const.LABEL_SUFFIX.length());
          while ((pos < v.size()) &&
                 (str.compareTo((String)v.elementAt(pos)) > 0)) pos++;
          v.insertElementAt(str,pos);
        } // end if
      } // end while
      // copy vector to result array
      result = new String[v.size()];
      for (int i=0; i<v.size(); i++) result[i] = (String)v.elementAt(i);
    } catch (IOException excep) {
      excep.printStackTrace();
    }
    System.gc();
    return result == null ? new String[0] : result;
  } // end getWorkLoadKeys

  /** Return a default file to display in the save file dialog. */
  public File getDefaultFile() {
    File file = super.getDefaultFile();

    if ((file != null) &&
        (file.getName().indexOf('*') < 0)) return file;
    if ((menuExt == null) || (menuExt.length < 1)) return null;
    return new File("*" + menuExt[0]);
  } // end getDefaultFile

  public boolean accept(File dir, String name) {
    if (menuExt == null) return true;
    for (int i=0; i<menuExt.length; i++)
      if (name.endsWith(menuExt[i])) return true;
    return false;
  } // end accept

  private final String getSetting(String key) {
    return ResourceUtil.getResStringOrKey(resources,key);
  } // end getSetting

  private final void init() {
    String str;

    // initalize the filename filter
    str = ResourceUtil.getResourceString(resources,MENU_EXT);
    if (str != null) menuExt = ResourceUtil.splitToWords(str);
      else System.err.println("MenuEditorPanel: " + MENU_EXT +
                              " not in resource file");
  } // end init

  // inner classes
  /** Insert a new menu item. */
  class InsItemAction extends AbstractAction {

   // constructors
    InsItemAction() {
      super(INSITEM_ACTION);
    } // end constructor InsItemAction

    // methods
    public void actionPerformed(ActionEvent e) {
      String[] aStr  = getWorkloadKeys();
      InsItemDlg dlg = new InsItemDlg(getFrame(),aStr);

      dlg.show();
      if (!dlg.isCanceled())
        getEditor().replaceSelection(dlg.getItemKeys());
      // end if
    } // end actionPerformed

  } // end InsItemAction

  /** Insert a new menu. */
  class InsMenuAction extends AbstractAction {

   // constructors
    InsMenuAction() {
      super(INSMENU_ACTION);
    } // end constructor InsMenuAction

    // methods
    public void actionPerformed(ActionEvent e) {
      String[] aStr  = getWorkloadKeys();
      String title   = ResourceUtil.getResStringOrKey(resources,MENUDLG_TITLE);
      InsMenuDlg dlg = new InsMenuDlg(getFrame(),aStr,title,true,true);

      dlg.show();
      if (!dlg.isCanceled())
        getEditor().replaceSelection(dlg.getMenuKeys());
      // end if
    } // end actionPerformed

  } // end InsMenuAction

  /** Insert a new menubar. */
  class InsMenubarAction extends AbstractAction {

   // constructors
    InsMenubarAction() {
      super(INSMENUBAR_ACTION);
    } // end constructor InsMenubarAction

    // methods
    public void actionPerformed(ActionEvent e) {
      String[] aStr  = getWorkloadKeys();
      String title   = ResourceUtil.getResStringOrKey(resources,MENUBARDLG_TITLE);
      InsMenuDlg dlg = new InsMenuDlg(getFrame(),aStr,title,false,false);

      dlg.show();
      if (!dlg.isCanceled())
        getEditor().replaceSelection(dlg.getMenubarKeys());
      // end if
    } // end actionPerformed

  } // end InsMenubarAction

  /** Insert a new toolbar. */
  class InsToolbarAction extends AbstractAction {

   // constructors
    InsToolbarAction() {
      super(INSTOOLBAR_ACTION);
    } // end constructor InsToolbarAction

    // methods
    public void actionPerformed(ActionEvent e) {
      String[] aStr  = getWorkloadKeys();
      String title   = ResourceUtil.getResStringOrKey(resources,TOOLBARDLG_TITLE);
      InsMenuDlg dlg = new InsMenuDlg(getFrame(),aStr,title,true,false);

      dlg.show();
      if (!dlg.isCanceled())
        getEditor().replaceSelection(dlg.getToolbarKeys());
      // end if
    } // end actionPerformed

  } // end InsToolbarAction

  /** Preview the menu. */
  class PreviewAction extends AbstractAction {

   // constructors
    PreviewAction() {
      super(PREVIEW_ACTION);
    } // end constructor PreviewAction

    // methods
    public void actionPerformed(ActionEvent e) {
      PreviewDlg dlg = new PreviewDlg(getFrame());

      dlg.show();
      if (!dlg.isCanceled()) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream(4096);
        OutputStreamWriter osw     = new OutputStreamWriter(baos);
        ByteArrayInputStream bais;
        Document doc;
        Object obj;

        dlg.readResults();
        try {
          doc = getEditor().getDocument();
          obj = doc.getProperty(CodeEditorKit.CHANGED_PROPERTY);
          getEditor().write(osw);
          // avoid, that the document is marked as not changed on reading it
          doc.putProperty(CodeEditorKit.CHANGED_PROPERTY,obj);
          osw.close();
          baos.close();
          bais = new ByteArrayInputStream(baos.toByteArray());
          osw  = null;
          baos = null;
          if (useSwing) {
            JMenuTestFrame mtf;
            String title = ResourceUtil.getResStringOrKey(resources,SWINGFRAME_TITLE);

            mtf = new JMenuTestFrame(title,getFrame().getIconImage(),bais,
                                     menuKey,toolbarKey,useImage,
                                     getProjectDirectory());
            mtf.setSize(getFrame().getSize());
            FrameUtil.shuffleLocation(mtf);
            mtf.setVisible(true);
          }
          else {
            MenuTestFrame mtf;
            String title = ResourceUtil.getResStringOrKey(resources,AWTFRAME_TITLE);

            mtf = new MenuTestFrame(title,getFrame().getIconImage(),
                                    bais,menuKey,toolbarKey);
            mtf.setSize(getFrame().getSize());
            FrameUtil.shuffleLocation(mtf);
            mtf.setVisible(true);
          } // end if
        } catch (IOException excep) {
          excep.printStackTrace();
        }
        System.gc();
      } // end if
    } // end actionPerformed

  } // end PreviewAction

  /** A dialog that builds the properties for a menu item. */
  class InsItemDlg extends BaseDialog
    implements TextListener,ActionListener {

    // instance variable declarations
    protected SmartTextField tfKey;
    protected Choice choice;
    protected TextField tfLabel;
    protected TextField tfAction;
    protected TextField tfAccel;
    protected Checkbox cbCtrl;
    protected Checkbox cbAlt;
    protected Checkbox cbShift;
    protected TextField tfImage;
    protected TextField tfTooltip;

    // constructors
    /** Create a new InsItemDlg object. */
    public InsItemDlg(Frame parent, String[] strings) {
      super(parent,true);

      Panel panel = getContentPanel();
      Label label;
      String[] options;
      int i;

      setTitle(ResourceUtil.getResStringOrKey(resources,ITEMDLG_TITLE));
      createButtons(ResourceUtil.getResStringOrKey(resources,ITEMDLG_BUTTONS));

      GridBagLayout gridbag          = new GridBagLayout();
      GridBagConstraints constraints = new GridBagConstraints();
      panel.setLayout(gridbag);
      constraints.insets     = new Insets(10,6,10,6);
      constraints.anchor     = GridBagConstraints.WEST;
      constraints.weightx    = 3.0;
      constraints.weighty    = 1.0;
      constraints.gridwidth  = 1;
      constraints.gridheight = 1;
      label = new Label(getSetting(ITEMDLG_KEYLABEL));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets = new Insets(10,0,10,6);
      tfKey = new SmartTextField(strings,true,15);
      tfKey.addTextListener(this);
      gridbag.setConstraints(tfKey,constraints);
      panel.add(tfKey);
      label = new Label(getSetting(ITEMDLG_TYPELABEL));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.gridwidth  = GridBagConstraints.REMAINDER;
      choice  = new Choice();
      options = ResourceUtil.splitToWords(getSetting(ITEMDLG_CHOICE));
      for (i=0; i<options.length; i++) choice.add(options[i]);
      gridbag.setConstraints(choice,constraints);
      panel.add(choice);
      constraints.insets    = new Insets(0,6,10,6);
      constraints.gridwidth = 1;
      label = new Label(getSetting(ITEMDLG_LABELLABEL));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets     = new Insets(0,0,10,6);
      tfLabel = new TextField("",15);
      gridbag.setConstraints(tfLabel,constraints);
      panel.add(tfLabel);
      label = new Label(getSetting(ITEMDLG_ACTIONLABEL));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      tfAction = new TextField("",15);
      gridbag.setConstraints(tfAction,constraints);
      panel.add(tfAction);
      constraints.insets     = new Insets(0,6,10,6);
      constraints.gridwidth  = 1;
      constraints.gridheight = GridBagConstraints.RELATIVE;
      label = new Label(getSetting(ITEMDLG_ACCELLABEL));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets = new Insets(0,0,10,6);
      tfAccel = new TextField("",15);
      gridbag.setConstraints(tfAccel,constraints);
      panel.add(tfAccel);
      constraints.weightx = 2.0;
      cbCtrl = new Checkbox(getSetting(ITEMDLG_CTRLLABEL),true);
      gridbag.setConstraints(cbCtrl,constraints);
      panel.add(cbCtrl);
      cbAlt = new Checkbox(getSetting(ITEMDLG_ALTLABEL),false);
      gridbag.setConstraints(cbAlt,constraints);
      panel.add(cbAlt);
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      cbShift = new Checkbox(getSetting(ITEMDLG_SHIFTLABEL),false);
      gridbag.setConstraints(cbShift,constraints);
      panel.add(cbShift);
      constraints.weightx    = 3.0;
      constraints.insets     = new Insets(0,6,10,6);
      constraints.gridwidth  = 1;
      constraints.gridheight = GridBagConstraints.REMAINDER;
      label = new Label(getSetting(ITEMDLG_IMAGELABEL));
      gridbag.setConstraints(label,constraints);
      panel.add(label);

      Panel p = new Panel(new FlowLayout(FlowLayout.LEFT,0,0));
      Button b = new Button("^");

      constraints.insets = new Insets(0,0,10,6);
      tfImage = new TextField("",12);
      p.add(tfImage);
      b.addActionListener(this);
      p.add(b);
      gridbag.setConstraints(p,constraints);
      panel.add(p);
      label = new Label(getSetting(ITEMDLG_TOOLTIPLABEL));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      tfTooltip = new TextField("",15);
      gridbag.setConstraints(tfTooltip,constraints);
      panel.add(tfTooltip);
      textValueChanged(null);
      pack();
      center(parent);
      setResizable(false);
    } // end constructor InsItemDlg

    public String getItemKeys() {
      String key = tfKey.getText().trim();
      String str = tfLabel.getText().trim();
      String res;

      res = "# " + key + " menu item\n";
      if (str.equals("")) str = key;
      res += key + MenuPreview.Const.LABEL_SUFFIX + " = " + str + "\n";
      switch (choice.getSelectedIndex()) {
        case 1:
          res += (key + MenuPreview.Const.TYPE_SUFFIX + " = " +
                  MenuPreview.Const.TYPE_CHECKBOX + "\n");
        break;
        case 2:
          res += (key + MenuPreview.Const.TYPE_SUFFIX + " = " +
                  MenuPreview.Const.TYPE_MENU + "\n");
        default:
      } // end switch
      str = tfAction.getText().trim();
      if (!str.equals(""))
        res += (key + MenuPreview.Const.ACTION_SUFFIX + " = " + str + "\n");
      // end if
      str = tfAccel.getText().trim();
      if (!str.equals("")) {
        res += (key + MenuPreview.Const.ACCEL_SUFFIX + " = " + str);
        if (!cbCtrl.getState()) res += " CTRL_MASK";
        if (cbAlt.getState())   res += " ALT_MASK";
        if (cbShift.getState()) res += " SHIFT_MASK";
        res += "\n";
      } // end if
      str = tfImage.getText().trim().replace(File.separatorChar,'/');
      if (!str.equals("")) {
        res += (key + MenuPreview.Const.IMAGE_SUFFIX + " = " + str + "\n");
      } // end if
      str = tfTooltip.getText().trim();
      if (!str.equals(""))
        res += (key + MenuPreview.Const.TOOLTIP_SUFFIX + " = " + str + "\n");
      // end if
      return res;
    } // end getItemKeys

    // --- TextListener ---------------------------------------------
    /** Invoked when the value of the text has changed. */
    public void textValueChanged(TextEvent e) {
      setButtonEnabled(0,!tfKey.getText().trim().equals(""));
    } // end textValueChanged

    // --- ActionListener ---------------------------------------------
    /** Invoked when an action occurs. */
    public void actionPerformed(ActionEvent e) {
      String name,path;
      FileDialog dlg;
      File result = null;

      dlg = new FileDialog(getFrame());
      dlg.setDirectory(getProjectDirectory());
      dlg.setFile("*.gif");
      dlg.show();
      name = dlg.getFile();
      if (name != null) {
        path = dlg.getDirectory();
        if (path == null) result = new File(name);
          else result = new File(path,name);
      } // end if
      if (result == null) return;
      path = result.getAbsolutePath();
      if (!path.startsWith(getProjectDirectory())) {
        JOptionPane.showMessageDialog(getFrame(),
          getSetting(ITEMDLG_WRONGPATHMSG),getTitle(),
          JOptionPane.ERROR_MESSAGE);
        return;
      } // end if
      path = StringUtil.relativePath(path,getProjectDirectory());
      path = path.replace(File.separatorChar,'/');
      tfImage.setText(path);
    } // end actionPerformed

  } // end InsItemDlg

  /** A dialog that builds the properties for a menu. */
  class InsMenuDlg extends BaseDialog
    implements TextListener, ItemListener, ActionListener {

    // instance variable declarations
    protected SmartTextField tfKey;
    protected TextField tfLabel;
    protected SmartTextField tfItemKey;
    protected Button addBtn;
    protected Button delBtn;
    protected Button separBtn;
    protected List list;

    // constructors
    /** Create a new InsMenuDlg object. */
    public InsMenuDlg(Frame parent, String[] strings, String title,
                      boolean useSeparBtn, boolean useLabel) {
      super(parent,true);

      Panel panel = getContentPanel();
      Panel subPanel;
      Label label;

      setTitle(title);
      createButtons(ResourceUtil.getResStringOrKey(resources,MENUDLG_BUTTONS));

      GridBagLayout gridbag          = new GridBagLayout();
      GridBagConstraints constraints = new GridBagConstraints();
      panel.setLayout(gridbag);
      constraints.insets     = new Insets(10,6,10,6);
      constraints.anchor     = GridBagConstraints.WEST;
      constraints.weightx    = 4.0;
      constraints.weighty    = 1.0;
      constraints.gridwidth  = 1;
      constraints.gridheight = 1;
      label = new Label(getSetting(MENUDLG_KEYLABEL));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets    = new Insets(10,0,10,6);
      constraints.weightx   = 5.0;
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      tfKey = new SmartTextField(strings,true,15);
      gridbag.setConstraints(tfKey,constraints);
      panel.add(tfKey);
      if (useLabel) {
        constraints.insets    = new Insets(0,6,10,6);
        constraints.weightx   = 4.0;
        constraints.gridwidth = 1;
        label = new Label(getSetting(MENUDLG_LABELLABEL));
        gridbag.setConstraints(label,constraints);
        panel.add(label);
        constraints.insets    = new Insets(0,0,10,6);
        constraints.weightx   = 5.0;
        constraints.gridwidth = GridBagConstraints.REMAINDER;
        tfLabel = new TextField("",15);
        gridbag.setConstraints(tfLabel,constraints);
        panel.add(tfLabel);
      } // end if
      constraints.insets     = new Insets(0,6,10,6);
      constraints.weightx    = 4.0;
      constraints.weighty    = 3.0;
      constraints.gridwidth  = 1;
      constraints.gridheight = GridBagConstraints.REMAINDER;
      subPanel = new Panel(new GridLayout(0,1,0,10));
      tfItemKey = new SmartTextField(strings,true,15);
      tfItemKey.addTextListener(this);
      subPanel.add(tfItemKey);
      label = new Label("");
      subPanel.add(label);
      if (useSeparBtn) {
        separBtn = new Button(getSetting(MENUDLG_SEPARBTN));
        separBtn.addActionListener(this);
        subPanel.add(separBtn);
      }
      else {
        subPanel.add(new Label(""));
      } // end if
      gridbag.setConstraints(subPanel,constraints);
      panel.add(subPanel);
      constraints.insets    = new Insets(0,0,10,6);
      constraints.weightx   = 1.0;
      constraints.gridwidth = GridBagConstraints.RELATIVE;
      subPanel = new Panel(new GridLayout(0,1,0,10));
      addBtn = new Button(">");
      addBtn.setEnabled(false);
      addBtn.addActionListener(this);
      subPanel.add(addBtn);
      delBtn = new Button("<");
      delBtn.setEnabled(false);
      delBtn.addActionListener(this);
      subPanel.add(delBtn);
      label = new Label("");
      subPanel.add(label);
      gridbag.setConstraints(subPanel,constraints);
      panel.add(subPanel);
      constraints.weightx   = 4.0;
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill      = GridBagConstraints.VERTICAL;
      list = new List();
      list.add("");
      list.select(0);
      list.addItemListener(this);
      list.addActionListener(this);
      gridbag.setConstraints(list,constraints);
      panel.add(list);
      pack();
      center(parent);
      setResizable(false);
    } // end constructor InsMenuDlg

    public String getMenuKeys() {
      String key = tfKey.getText().trim();
      String str,res;

      res  = "# " + key + " menu\n";
      res += (key + " =");
      for (int i=0; i<list.getItemCount(); i++)
        res += (" " + list.getItem(i));
      // end for
      str = tfLabel.getText().trim();
      if (str.equals("")) str = key;
      res += ("\n" + key + MenuPreview.Const.LABEL_SUFFIX + " = " + str + "\n");
      return res;
    } // end getMenuKeys

    public String getMenubarKeys() {
      return getBar("menubar");
    } // end getMenubarKeys

    public String getToolbarKeys() {
      return getBar("toolbar");
    } // end getToolbarKeys

    protected String getBar(String comment) {
      String key = tfKey.getText().trim();
      String res = "# " + key + " " + comment + "\n";

      res += (key + " =");
      for (int i=0; i<list.getItemCount(); i++)
        res += (" " + list.getItem(i));
      // end for
      res += "\n";
      return res;
    } // end getBar

    // --- TextListener ---------------------------------------------
    /** Invoked when the value of the text has changed. */
    public void textValueChanged(TextEvent e) {
      if (e.getSource() == tfItemKey) {
        addBtn.setEnabled(!tfItemKey.getText().trim().equals(""));
      } // end if
    } // end textValueChanged

    // --- ItemListener ---------------------------------------------
    /** Invoked when an item's state has been changed. */
    public void itemStateChanged(ItemEvent e) {
      int index = list.getSelectedIndex();

      if (index < 0) getToolkit().beep();
        else delBtn.setEnabled(!list.getItem(index).equals(""));
    } // end itemStateChanged

    // --- ActionListener -------------------------------------------
    /** Invoked when an action occurs. */
    public void actionPerformed(ActionEvent e) {
      int index = list.getSelectedIndex();

      if (index < 0) {
        getToolkit().beep();
        return;
      } // end if
      if ((e.getSource() == delBtn) || (e.getSource() == list)) {
        if (list.getItem(index).equals("")) {
          getToolkit().beep();
        }
        else {
          list.remove(index);
          list.select(index);
        } // end if
      } else if (e.getSource() == addBtn) {
        list.add(tfItemKey.getText().trim(),index);
        tfItemKey.setText("");
      } else { // The add separator button
        list.add("-",index);
      } // end if
    } // end actionPerformed

  } // end InsMenuDlg

  /** A dialog to specify the menu key to build a preview of the menu. */
  class PreviewDlg extends BaseDialog
    implements TextListener, ItemListener {

    // instance variable declarations
    protected TextField tfMenuKey;
    protected Checkbox cbUseSwing;
    protected Checkbox cbUseImage;
    protected Label lToolBarKey;
    protected TextField tfToolBarKey;

    // constructors
    /** Create a new PreviewDlg object. */
    public PreviewDlg(Frame parent) {
      super(parent,true);

      Panel panel = getContentPanel();
      Label label;

      setTitle(ResourceUtil.getResStringOrKey(resources,PREVIEWDLG_TITLE));
      createButtons(ResourceUtil.getResStringOrKey(resources,PREVIEWDLG_BUTTONS));

      GridBagLayout gridbag          = new GridBagLayout();
      GridBagConstraints constraints = new GridBagConstraints();
      panel.setLayout(gridbag);
      constraints.insets     = new Insets(10,6,10,6);
      constraints.anchor     = GridBagConstraints.WEST;
      constraints.weightx    = 1.0;
      constraints.weighty    = 1.0;
      constraints.gridwidth  = 1;
      constraints.gridheight = 1;
      label = new Label(getSetting(PREVIEWDLG_MENULABEL));
      gridbag.setConstraints(label,constraints);
      panel.add(label);
      constraints.insets    = new Insets(10,0,10,6);
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      tfMenuKey = new TextField(menuKey,15);
      tfMenuKey.addTextListener(this);
      gridbag.setConstraints(tfMenuKey,constraints);
      panel.add(tfMenuKey);
      constraints.insets = new Insets(10,6,10,6);
      cbUseSwing = new Checkbox(getSetting(PREVIEWDLG_SWINGLABEL),useSwing);
      cbUseSwing.addItemListener(this);
      gridbag.setConstraints(cbUseSwing,constraints);
      panel.add(cbUseSwing);
      constraints.gridheight = GridBagConstraints.RELATIVE;
      cbUseImage = new Checkbox(getSetting(PREVIEWDLG_IMAGELABEL),useImage);
      gridbag.setConstraints(cbUseImage,constraints);
      panel.add(cbUseImage);
      constraints.gridwidth  = 1;
      constraints.gridheight = GridBagConstraints.REMAINDER;
      lToolBarKey = new Label(getSetting(PREVIEWDLG_TOOLLABEL));
      gridbag.setConstraints(lToolBarKey,constraints);
      panel.add(lToolBarKey);
      constraints.insets    = new Insets(10,0,10,6);
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      tfToolBarKey = new TextField(toolbarKey,15);
      tfToolBarKey.addTextListener(this);
      gridbag.setConstraints(tfToolBarKey,constraints);
      panel.add(tfToolBarKey);
      // update the enabled state of some controls and the OK button
      itemStateChanged(null);
      textValueChanged(null);
      pack();
      center(parent);
      setResizable(false);
    } // end constructor PreviewDlg;

    // methods
    void readResults() {
      menuKey    = tfMenuKey.getText();
      useSwing   = cbUseSwing.getState();
      useImage   = cbUseImage.getState();
      toolbarKey = tfToolBarKey.getText();
    } // end readResults

    // --- TextListener ---------------------------------------------
    /** Invoked when the value of the text has changed. */
    public void textValueChanged(TextEvent e) {
      setButtonEnabled(0,!tfMenuKey.getText().trim().equals("") ||
                         !tfToolBarKey.getText().trim().equals(""));
    } // end textValueChanged

    // --- ItemListener ---------------------------------------------
    /** Invoked when an item's state has been changed. */
    public void itemStateChanged(ItemEvent e) {
      cbUseImage.setEnabled(cbUseSwing.getState());
    } // end itemStateChanged

  } // end PreviewDlg;

} // end MenuEditorPanel

/* ******************************************************************
   end of file
******************************************************************* */
