#define COMST "/*"
#define COMEN "*/"
#pragma unquote COMST
#pragma unquote COMEN
#pragma getstring PACKAGE Package
COMST ******************************************************************

   class      : ToolBarLoader
#define __NAME__ ToolBarLoader
#define __EXT__ .java
   date       : __DATE__
#ifdef __COMPILER__
   compiler : __COMPILER__
#endif

******************************************************************* COMEN

#ifdef PACKAGE
package PACKAGE;
  #define __RELPATH__ PACKAGE

#endif
/* ******************************************************************
   import statements
 *********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/* ******************************************************************
   top level class declaration
 *********************************************************************/

/**
 * Builds up a AWT toolbar by reading the settings from a resource file.
 * A AWT toolbar is a simple Panel with Buttons, that can't get the focus.
 * It's intended to use this class for loading toolbar's created with
 * Magic Menu.
 *
 * @author Puehringer Edgar
 */
class ToolBarLoader {

  // class variable declarations
  /**
   * Suffix applied to the key used in resource file
   * lookups for a label.
   */
  public static final String LABEL_SUFFIX = "Label";

  /**
   * Suffix applied to the key used in resource file
   * lookups for an action.
   */
  public static final String ACTION_SUFFIX = "Action";

  /** The escape character to identify a mnemonic in a label. */
  public static final char MNEMONIC_ESCAPE = '&';

   /** The blanks for the label between separated AWT toolbar components. */
   public static final String STRUT_BLANKS = " ";

  // instance variable declarations
  private ResourceBundle resources;
  private ActionListener defaultAction;

  // constructors
  /**
   * Create a new ToolBarLoader object.
   *
   * @param resources A resources bundle containing the toolbar settings.
   */
  ToolBarLoader(ResourceBundle resources) {
     this.resources = resources;
     defaultAction  = null;
  } // end constructor ToolBarLoader

  // class methods
  /** Filters the mnemonic identifiers out of a label string. */
  private static String filterMnemonic(String label, char[] ch) {
    int index = label.indexOf(MNEMONIC_ESCAPE);

    if (index < 0) return label;
    if ((index + 1) < label.length()) {
      char c = label.charAt(index + 1);

      if (c != MNEMONIC_ESCAPE) ch[0] = c;
      return label.substring(0,index) + c +
        filterMnemonic(label.substring(index + 2),ch);
    }
    else {
      return label.substring(0,index);
    } // end if
  } // end filterMnemonic

  // methods
  /**
   * Returns the default action listener.  This ActionListener is
   * registered at every toolbar item.
   */
  public ActionListener getDefaultActionListener() {
    return defaultAction;
  } // end getDefaultActionListener

  /**
   * Set the default action listener.  This action listener is registered at
   * every toolbar item. If a ActionListener is set, this should be done
   * before the first call to createToolBar, createTool or createMenuItem.
   *
   * @see #createToolBar
   * @see #createTool
   * @see #createToolBarButton
   */
  public void setDefaultActionListener(ActionListener al) {
    defaultAction = al;
  } // end setDefaultActionListener

  /**
   * Create a button to go inside of the toolbar.
   *
   * @param key The key in the resource file to serve as the basis
   *            of lookups.
   */
  protected Button createToolBarButton(String key) {
    String label = getResourceString(key + LABEL_SUFFIX);
    String astr  = getResourceString(key + ACTION_SUFFIX);
    Button b;
    char[] mnemonic = new char[1];

    if (label == null) {
      reportError("Label property for toolbar button key \"" +
                  key + "\" not found");
      return null;
    } // end if
    b = new Button(filterMnemonic(label,mnemonic)) {

      public boolean isFocusTraversable() {
        return false;
      } // end isFocusTraversable

    };
    if(astr == null) astr = key;
    b.setActionCommand(astr);

    ActionListener a = getAction(astr);

    if(a != null) {
      b.addActionListener(a);
    }
    else {
      b.setEnabled(false);
    } // end if
    return b;
  } // end createToolBarButton

  /**
   * Return the default action listener.  This Method should be overwritten,
   * if not all tollbar buttons should register the same action listener.
   *
   * @param cmd The action command of the associated toolbar button.
   */
  protected ActionListener getAction(String cmd) {
    return defaultAction;
  } // end getAction

  /**
   * Read a string from the resource file containing the toolbar
   * definition.
   */
  protected String getResourceString(String key) {
    String str;

    try {
      str = resources.getString(key);
    } catch (MissingResourceException mre) {
      str = null;
    }
    return str;
  } // end getResourceString

  /**
   * Take the given string and chop it up into a series
   * of strings on whitespace boundries. This is useful
   * for trying to get an array of strings out of the
   * resource file.
   */
  protected String[] tokenize(String input) {
    Vector v = new Vector();
    StringTokenizer t = new StringTokenizer(input);
    String cmd[];

    while(t.hasMoreTokens()) v.addElement(t.nextToken());
    cmd = new String[v.size()];
    for(int i = 0;i < cmd.length;i++)
      cmd[i] = (String)v.elementAt(i);
    // end for
    return cmd;
  } // end tokenize

  /**
   * Create the toolbar for the specified key.  This pulls the
   * definition of the toolbar from the associated resource bundle.
   */
  protected Panel createToolBar(String key) {
    Panel tb = new Panel(new FlowLayout(FlowLayout.LEFT));
    String tbDesc = getResourceString(key);
    String[] toolKeys;

    if (tbDesc == null) {
      reportError("ToolBar \"" + key + "\" not found or empty");
      return null;
    } // end if
    toolKeys = tokenize(tbDesc);
    for (int i=0; i<toolKeys.length; i++) {
      if(toolKeys[i].equals("-")) {
        tb.add(new Label(STRUT_BLANKS));
      }
      else {
        Component comp = createTool(toolKeys[i]);

        if (comp != null) tb.add(comp);
      } // end if
    } // end for
    tb.setBackground(SystemColor.control);
    return tb;
  } // end createToolBar

  /**
   * Hook through which every toolbar item is created.  This method
   * can be used to integrate custom gui elements into the toolbar.
   */
  protected Component createTool(String key) {
    return createToolBarButton(key);
  } // end createTool

  /**
   * This method will be called, if some errors occur.  By default this
   * will cause an output to stderr.
   */
  protected void reportError(String text) {
    System.err.println("ToolBarLoader: " + text + ".");
  } // end reportError

} // end ToolBarLoader

/* ******************************************************************
   end of file
 *********************************************************************/
